
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : sse.scm
;; DESCRIPTION : The Standalone Server Example
;; AUTHOR      : Henri Lesourd, Uni Saarbrucken, August 2006.
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(texmacs-module (sse))

;; Utils
(define (tm-valid-inner-path? p)
  (and (pair? p) (> (length p) 2)))

(define (tm-inner-atomic-path? p)
  (and (tm-valid-inner-path? p)
       (not (tm-eol-path? p))
       (or (tree-atomic? (path->tree (cDr p)))
          ;(eq? (tree-label (path->tree p)) 'string)
       )))

(define (tm-eol-path? p)
  (and (tm-valid-inner-path? p)
       (<= (tree-arity (path->tree (cDr p))) (cAr p))))

(define (tm-adjust-path path)
  (if (or (tm-inner-atomic-path? path)
          (tm-eol-path? path)
      )
      (set! path (cDr path))
  )
  path)

(define (tm-upwards-path p tags nottags)
  (set! p (tm-adjust-path p))
  (if (in? (tree-label (path->tree p)) tags)
      p
      (if (in? (tree-label (path->tree p)) nottags)
          #f
          (if (> (length p) 2)
              (tm-upwards-path (cDr p) tags nottags)
              #f))))

;; Client->Server functions
(load "server.scm")
(define (send-chunk path)
  (with p (tm-upwards-path path '(sse-document) '())
     (if p
     (with msg (tree->stree (path->tree p))
        (display "chunk=")
        (write msg)
        (newline)
        (out (send-msg msg))))))

;; Keyboard shortcuts 
(define (in-sse?)
  (tm-upwards-path (cDr (cursor-path)) '(sse-document) '()))

;; Initialization
(tm-define (sse-initialize)
  (display* "Loading SSE...\n")
  (kbd-map
    (:mode in-sse?)
    ("C-return" (send-chunk (cursor-path)))
    ("C-S-return" (out (send-msg '(shutdown)))) ;; Shutdowning the server
  )
  (out (send-msg `(connect ,(getpid)))) ;; Initial connect
)
